@@ -2,38 +2,38 @@ use strict;
 use Encode qw(decode_utf8 encode_utf8);
 use Unicode::LineBreak qw(:all);
 
-$Unicode::LineBreak::Config = {
-    CharactersMax => 998,
-    ColumnsMin => 0,
-    ColumnsMax => 76,
+@Unicode::LineBreak::Config = (
+    CharMax => 998,
+    ColMax => 76,
+    ColMin => 0,
     Context => 'NONEASTASIAN',
+    EAWidth => [[0x302E, 0x302F] => EA_Z()], # 6.1.0: Changed from Mn to Mc.
     Format => 'SIMPLE',
     HangulAsAL => 'NO',
+    LBClass => undef,
     LegacyCM => "YES",
     Newline => "\n",
-    SizingMethod => "UAX11",
-    TailorEA => [],
-    TailorLB => [],
-    UrgentBreaking => undef,
-    UserBreaking => [],
-};
+    Prep => undef,
+    Sizing => "UAX11",
+    Urgent => undef,
+);
 
 sub dotest {
     my $in = shift;
     my $out = shift;
 
-    open IN, "<testin/$in.in" or die "open: $!";
+    open IN, "<test-data/$in.in" or die "open: $!";
     my $instring = decode_utf8(join '', <IN>);
     close IN;
     my $lb = Unicode::LineBreak->new(@_);
     my $broken = encode_utf8($lb->break($instring));
 
     my $outstring = '';
-    if (open OUT, "<testin/$out.out") {
+    if (open OUT, "<test-data/$out.out") {
 	$outstring = join '', <OUT>;
 	close OUT;
     } else {
-	open XXX, ">testin/$out.xxx";
+	open XXX, ">test-data/$out.xxx";
 	print XXX $broken;
 	close XXX;
     }
@@ -47,7 +47,7 @@ sub dotest_partial {
     my $len = shift;
 
     my $lb = Unicode::LineBreak->new(@_);
-    open IN, "<testin/$in.in" or die "open: $!";
+    open IN, "<test-data/$in.in" or die "open: $!";
     my $instring = decode_utf8(join '', <IN>);
     close IN;
 
@@ -64,11 +64,11 @@ sub dotest_partial {
     $broken .= encode_utf8($lb->break_partial(undef));
 
     my $outstring = '';
-    if (open OUT, "<testin/$out.out") {
+    if (open OUT, "<test-data/$out.out") {
 	$outstring = join '', <OUT>;
 	close OUT;
     } else {
-	open XXX, ">testin/$out.xxx";
+	open XXX, ">test-data/$out.xxx";
 	print XXX $broken;
 	close XXX;
     }
@@ -80,18 +80,18 @@ sub dotest_array {
     my $in = shift;
     my $out = shift;
 
-    open IN, "<testin/$in.in" or die "open: $!";
+    open IN, "<test-data/$in.in" or die "open: $!";
     my $instring = decode_utf8(join '', <IN>);
     close IN;
     my $lb = Unicode::LineBreak->new(@_);
     my @broken = map { encode_utf8("$_") } $lb->break($instring);
 
     my @outstring = ();
-    if (open OUT, "<testin/$out.out") {
+    if (open OUT, "<test-data/$out.out") {
 	@outstring = <OUT>;
 	close OUT;
     } else {
-	open XXX, ">testin/$out.xxx";
+	open XXX, ">test-data/$out.xxx";
 	print XXX join '', @broken;
 	close XXX;
     }
